// Client, client application for the serialmon board.
//
// Copyright (C) 2008, Frank Tkalcevic, www.franksworkshop.com

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace Client
{
    class CircularBuffer<T>
    {
        T[] items;
        int start;
        int end;

        public CircularBuffer( int nSize )
        {
            items = new T[nSize];
            start = 0;
            end = 0;
        }

        public T this[int index]
        {
            get
            {
                if (index > Count)
                    throw new ApplicationException("Indes out of range");
                int n = (start + index) % items.Length;
                return items[n];
            }
        }

        public void Add(T item)
        {
            items[end] = item;
            end = (end+1) % items.Length;
            if ( start == end )
                start = (start + 1) % items.Length;
        }

        public int Count
        {
            get
            {
                int n = end - start;
                if (n < 0)
                    n += items.Length;
                return n;
            }
        }

        public void Clear()
        {
            start = 0;
            end = 0;
        }

        public T[] ToArray()
        {
            T[] ret = new T[Count];
            for (int i = 0; i < Count; i++)
                ret[i] = this[i];
            return ret;
        }

    }
}
