// Client, client application for the serialmon board.
//
// Copyright (C) 2008, Frank Tkalcevic, www.franksworkshop.com

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace Client
{
    class DisplayModeFormattedText: DisplayMode
    {
        private Char nLastData = '\0';
        private int nWidest = 0;

        public DisplayModeFormattedText(CircularBuffer<ushort> buf, DisplayProperties props)
            : base( buf, props )
        {
        }

        private void AddDisplayData( DisplayData dd )
        {
            if (DisplayBuffer.Count <= nNextLinePosition)
            {
                DisplayBuffer.Add(new DisplayDataList());
            }

            if (dd.type == EDisplayDataType.Line1 || dd.type == EDisplayDataType.Line2)
            {
                char c = dd.data;

                if (!(c == '\r' || c == '\n' || c == '\t'))
                {
                    if (!IsPrint(c))
                        c = '.';
                    DisplayBuffer[nNextLinePosition][nNextCharPosition++] = new DisplayData(dd.type, c);
                    if (nNextCharPosition > nWidest)
                        nWidest = nNextCharPosition;
                }

                nDistanceFromStart++;

                if (c == '\n')
                {
                    if (nLastData != '\r')
                    {
                        nNextCharPosition = 0;
                        nNextLinePosition++;
                    }
                }
                else if (c == '\r')
                {
                    if (nLastData != '\n')
                    {
                        nNextCharPosition = 0;
                        nNextLinePosition++;
                    }
                }
                else if (c == '\t')
                {
                    while ( (nNextCharPosition+1) % 8 != 0 )
                        DisplayBuffer[nNextLinePosition][nNextCharPosition++] = new DisplayData(dd.type, ' ');
                }

                nLastData = c;
            }
        }

        protected override void AddText(string s)
        {
            // Not needed
        }

        // Add line data to the buffer
        protected override void AddData(int nLine, char c)
        {
            if ((nLine == 0 && DisplayProps.ShowLine1) || (nLine == 1 && DisplayProps.ShowLine2))
                AddDisplayData(new DisplayData(nLine == 0 ? EDisplayDataType.Line1 : EDisplayDataType.Line2, c));
        }

        // Add time to the buffer
        protected override void AddTime(byte nCount)
        {
            // Not supported
        }

        public override void RecomputeScrollRanges()
        {
            nLastData = '\0';
            nWidest = 0;
            base.RecomputeScrollRanges();
        }

        public override void UpdateScrollRanges(ushort data)
        {
            base.UpdateScrollRanges(data);
            nWidestLine = nWidest;
        }
    }
}
