// Client, client application for the serialmon board.
//
// Copyright (C) 2008, Frank Tkalcevic, www.franksworkshop.com

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace Client
{
    class DisplayModeHexAndText: DisplayMode
    {
        private const int ADDR_WIDTH = 6;
        private const int DATA_PER_LINE = 16;
        private const int LINE_WIDTH = ADDR_WIDTH + 1 + 3 * DATA_PER_LINE + 1 + 1 + DATA_PER_LINE;

        public DisplayModeHexAndText(CircularBuffer<ushort> buf, DisplayProperties props)
            : base( buf, props )
        {
        }

        private void AddDisplayData( DisplayData dd )
        {
            if (DisplayBuffer.Count <= nNextLinePosition)
            {
                DisplayBuffer.Add(new DisplayDataArray(LINE_WIDTH));

                String s = nDistanceFromStart.ToString("X6") + " ";
                AddText(s);
            }

            int nPos = nDistanceFromStart % DATA_PER_LINE;

            if (dd.type == EDisplayDataType.Line1 || dd.type == EDisplayDataType.Line2)
            {
                String sHex = ((int)dd.data).ToString("X2");
                char cText = dd.data;
                if (!IsPrint(cText))
                    cText = '.';

                int x = ADDR_WIDTH + 1 + nPos * 3 + nPos/8;
                DisplayBuffer[nNextLinePosition][x] = new DisplayData(dd.type, sHex.ToCharArray()[0]);
                DisplayBuffer[nNextLinePosition][x + 1] = new DisplayData(dd.type, sHex.ToCharArray()[1]);
                DisplayBuffer[nNextLinePosition][x + 2] = new DisplayData(dd.type, ' ');

                x = ADDR_WIDTH + 1 + DATA_PER_LINE * 3 + 1 + 1 + nPos;
                DisplayBuffer[nNextLinePosition][x] = new DisplayData(dd.type, cText);

                nDistanceFromStart++;
            }

            if (nDistanceFromStart % DATA_PER_LINE == 0 )
            {
                nNextCharPosition = 0;
                nNextLinePosition++;
            }
        }

        // Add a display string to the buffer - only used for time/address at the begining of the line, never causes a new line
        protected override void AddText(string s)
        {
            foreach (char c in s)
            {
                DisplayBuffer[nNextLinePosition][nNextCharPosition] = new DisplayData( EDisplayDataType.Text,c);
                nNextCharPosition++;
            }
        }

        // Add line data to the buffer
        protected override void AddData(int nLine, char c)
        {
            if ((nLine == 0 && DisplayProps.ShowLine1) || (nLine == 1 && DisplayProps.ShowLine2))
                AddDisplayData(new DisplayData(nLine == 0 ? EDisplayDataType.Line1 : EDisplayDataType.Line2, c));
        }

        // Add time to the buffer
        protected override void AddTime(byte nCount)
        {
            // Not supported
        }

        public override void UpdateScrollRanges(ushort data)
        {
            base.UpdateScrollRanges(data);
            nWidestLine = LINE_WIDTH;
        }

    }
}
