// Client, client application for the serialmon board.
//
// Copyright (C) 2008, Frank Tkalcevic, www.franksworkshop.com

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace Client
{
    class DisplayModeText: DisplayMode
    {
        public DisplayModeText(CircularBuffer<ushort> buf, DisplayProperties props)
            : base( buf, props )
        {
        }

        private void AddDisplayData( DisplayData dd )
        {
            if (DisplayBuffer.Count <= nNextLinePosition)
            {
                DisplayBuffer.Add(new DisplayDataArray(DisplayProps.CharactersPerLine));

                if (DisplayProps.ShowAddress || DisplayProps.ShowTime)
                {
                    String s = null;
                    if (DisplayProps.ShowAddress)
                        s = nDistanceFromStart.ToString("X6") + " ";
                    else
                    {
                        int tseconds = (int)(10.0 * (double)nDistanceFromStart / ((double)DisplayProps.Baud / 10.0));
                        int seconds = tseconds / 10;
                        tseconds = tseconds % 10;
                        int minutes = seconds / 60;
                        seconds = seconds % 60;

                        s = minutes.ToString("D2") + ":" + seconds.ToString("D2") + "." + tseconds.ToString("D1") + " ";
                    }

                    AddText(s);
                }
            }

            DisplayBuffer[nNextLinePosition][nNextCharPosition] = dd;
            nNextCharPosition++;
            nDistanceFromStart++;
            if (nNextCharPosition >= DisplayProps.CharactersPerLine)
            {
                nNextCharPosition = 0;
                nNextLinePosition++;
            }
        }

        // Add a display string to the buffer - only used for time/address at the begining of the line, never causes a new line
        protected override void AddText(string s)
        {
            foreach (char c in s)
            {
                DisplayBuffer[nNextLinePosition][nNextCharPosition] = new DisplayData(EDisplayDataType.Text, c);
                nNextCharPosition++;
            }
        }

        // Add line data to the buffer
        protected override void AddData(int nLine, char c)
        {
            if ((nLine == 0 && DisplayProps.ShowLine1) || (nLine == 1 && DisplayProps.ShowLine2))
            {
                if (!IsPrint(c))
                    c = '.';
                AddDisplayData(new DisplayData(nLine == 0 ? EDisplayDataType.Line1 : EDisplayDataType.Line2, c));
            }
        }

        // Add time to the buffer
        protected override void AddTime(byte nCount)
        {
            for ( int i = 0; i < nCount; i++ )
                AddDisplayData( new DisplayData( EDisplayDataType.Time, '\0' ) );
        }

    }
}
