// Client, client application for the serialmon board.
//
// Copyright (C) 2008, Frank Tkalcevic, www.franksworkshop.com

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;
using System.IO.Ports;


namespace Client
{
    class SerialPortDetails
    {
        private String sDevice;
        private String sFriendlyName;
        private String sPort;

        public SerialPortDetails(string sDevice, string sFriendlyName)
        {
            this.sDevice = sDevice;
            this.sFriendlyName = sFriendlyName;
            this.sPort = sFriendlyName.Split('(', ')')[1];
        }

        public String Device { get { return sDevice; } }
        public String FriendlyName { get { return sFriendlyName; } }
        public String Port { get { return sPort; } }
        public override string ToString()
        {
            return FriendlyName;
        }

    }

    class SerialPortEx: System.IO.Ports.SerialPort
    {
        public static unsafe List<SerialPortDetails> FindAllPorts()
        {
            List<SerialPortDetails> ports = new List<SerialPortDetails>();

            int hDevInfo;

            // Create a HDEVINFO with all present devices.
            hDevInfo = SetupInterface.SetupDiGetClassDevs(SetupInterface.GUID_DEVINTERFACE_COMPORT, null, 0, SetupInterface.DIGCF_PRESENT | SetupInterface.DIGCF_DEVICEINTERFACE);

            if (hDevInfo == SetupInterface.INVALID_HANDLE_VALUE)
            {
                // Insert error handling here.
                return ports;
            }

            for (int nIndex = 0; ; nIndex++)
            {
                SetupInterface.PSP_DEVICE_INTERFACE_DATA devInterfaceData = new SetupInterface.PSP_DEVICE_INTERFACE_DATA(0);
                bool bResult = SetupInterface.SetupDiEnumDeviceInterfaces(hDevInfo, null, SetupInterface.GUID_DEVINTERFACE_COMPORT, nIndex, out devInterfaceData);
                if (!bResult)
                {
                    int n = System.Runtime.InteropServices.Marshal.GetLastWin32Error();
                    break;
                }

                SetupInterface.PSP_DEVICE_INTERFACE_DETAIL_DATA detailData = new SetupInterface.PSP_DEVICE_INTERFACE_DETAIL_DATA(0);
                SetupInterface.PSP_DEVINFO_DATA devInfoData = new SetupInterface.PSP_DEVINFO_DATA(0);

                bResult = SetupInterface.SetupDiGetDeviceInterfaceDetail(hDevInfo, devInterfaceData, ref detailData, ref devInfoData);
                if (!bResult)
                {
                    UInt32 err = SetupInterface.GetLastError();
                }

                String sDevicePath = detailData.DevicePath;
                String sDeviceDesc = SetupInterface.GetDeviceRegistryProperty(hDevInfo, devInfoData, SetupInterface.SPDRP_DEVICEDESC);
                String sFriendlyName = SetupInterface.GetDeviceRegistryProperty(hDevInfo, devInfoData, SetupInterface.SPDRP_FRIENDLYNAME);
                String sLocationInfo = SetupInterface.GetDeviceRegistryProperty(hDevInfo, devInfoData, SetupInterface.SPDRP_LOCATION_INFORMATION);
                String sPhysicalName = SetupInterface.GetDeviceRegistryProperty(hDevInfo, devInfoData, SetupInterface.SPDRP_PHYSICAL_DEVICE_OBJECT_NAME);

                ports.Add( new SerialPortDetails(sDevicePath, sFriendlyName));
            }

            return ports;
        }

    }
}
