// Client, client application for the serialmon board.
//
// Copyright (C) 2008, Frank Tkalcevic, www.franksworkshop.com

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace Client
{
    class SetupInterface
    {
        private const int PATH_BUFFER_SIZE = 1024;
        public const int DIGCF_DEFAULT = 0x01;
        public const int DIGCF_PRESENT = 0x02;
        public const int DIGCF_ALLCLASSES = 0x04;
        public const int DIGCF_PROFILE = 0x08;
        public const int DIGCF_DEVICEINTERFACE = 0x10;
        static public Guid GUID_DEVINTERFACE_COMPORT = new Guid("{86E0D1E0-8089-11D0-9CE4-08003E301F73}");
        public const int INVALID_HANDLE_VALUE = 0;

        public const uint SPDRP_DEVICEDESC = 0x00000000;
        public const uint SPDRP_FRIENDLYNAME = 0x0000000C;
        public const uint SPDRP_LOCATION_INFORMATION = 0x0000000D;
        public const uint SPDRP_PHYSICAL_DEVICE_OBJECT_NAME = 0x0000000E;

        public const int ERROR_INSUFFICIENT_BUFFER = 122;

        public unsafe struct PSP_DEVICE_INTERFACE_DATA
        {
            public UInt32 cbSize;
            public fixed byte InterfaceClassGuid[16];
            public UInt32 Flags;
            public UInt32 Reserved;

            public PSP_DEVICE_INTERFACE_DATA(int n)
            {
                cbSize = (UInt32)sizeof(PSP_DEVICE_INTERFACE_DATA);
                Flags = 0;
                Reserved = 0;
            }
        };
        unsafe public struct PSP_DEVINFO_DATA
        {
            public UInt32 cbSize;
            public unsafe fixed byte ClassGuid[16];
            public UInt32 DevInst;
            public UInt32 Reserved;

            public PSP_DEVINFO_DATA(int n)
            {
                cbSize = (UInt32)sizeof(PSP_DEVINFO_DATA);
                DevInst = 0;
                Reserved = 0;
            }
        };

        [StructLayout(LayoutKind.Explicit, CharSet = CharSet.Unicode)]
        public unsafe struct PSP_DEVICE_INTERFACE_DETAIL_DATA
        {
            [FieldOffset(0)]
            public UInt32 cbSize;
            [FieldOffset(4)]
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = PATH_BUFFER_SIZE)]
            public String DevicePath;

            public PSP_DEVICE_INTERFACE_DETAIL_DATA(int n)
            {
                DevicePath = new string((char)0, 256);
                cbSize = 6;
            }
        };

        [DllImport("setupapi.dll")]
        private extern static int SetupDiGetClassDevs(byte[] guid, string sEnumerator, int hParent, int flags);

        public static int SetupDiGetClassDevs(Guid guid, string sEnumerator, int hParent, int flags)
        {
            return SetupDiGetClassDevs(guid.ToByteArray(), sEnumerator, hParent, flags);
        }

        [DllImport("setupapi.dll")]
        public static extern bool SetupDiDestroyDeviceInfoList(int hDeviceInfo);

        [DllImport("setupapi.dll", SetLastError = true)]
        private unsafe static extern bool SetupDiEnumDeviceInterfaces(int hDeviceInfoSet, PSP_DEVINFO_DATA* pDeviceInfoData, byte[] guid, int nIndex, out PSP_DEVICE_INTERFACE_DATA deviceInterfaceData);

        public unsafe static bool SetupDiEnumDeviceInterfaces(int hDeviceInfoSet, PSP_DEVINFO_DATA* pDeviceInfoData, Guid guidInterfaceClass, int nIndex, out PSP_DEVICE_INTERFACE_DATA deviceInterfaceData)
        {
            return SetupDiEnumDeviceInterfaces(hDeviceInfoSet, pDeviceInfoData, guidInterfaceClass.ToByteArray(), nIndex, out deviceInterfaceData);
        }

        [DllImport("Kernel32.dll")]
        public static extern UInt32 GetLastError();

        [DllImport("setupapi.dll", EntryPoint = "SetupDiGetDeviceInterfaceDetailW")]
        private unsafe static extern bool SetupDiGetDeviceInterfaceDetailW(int hDeviceInfoSet, ref PSP_DEVICE_INTERFACE_DATA DeviceInterfaceData, ref PSP_DEVICE_INTERFACE_DETAIL_DATA DeviceInterfaceDetailData, UInt32 nDeviceInterfaceDetailDataSize, out UInt32 nRequiredSize, ref PSP_DEVINFO_DATA DeviceInfoData);

        [DllImport("setupapi.dll", EntryPoint = "SetupDiGetDeviceInterfaceDetailW")]
        private unsafe static extern bool SetupDiGetDeviceInterfaceDetailW_Size(int hDeviceInfoSet, ref PSP_DEVICE_INTERFACE_DATA DeviceInterfaceData, int n, UInt32 nDeviceInterfaceDetailDataSize, out UInt32 nRequiredSize, int m);

        public unsafe static bool SetupDiGetDeviceInterfaceDetail(int hDeviceInfoSet, [In] PSP_DEVICE_INTERFACE_DATA DeviceInterfaceData, [In, Out] ref PSP_DEVICE_INTERFACE_DETAIL_DATA DeviceInterfaceDetailData, [In, Out] ref PSP_DEVINFO_DATA DeviceInfoData)
        {
            UInt32 nRequiredSize;
            bool b;
            b = SetupDiGetDeviceInterfaceDetailW_Size(hDeviceInfoSet, ref DeviceInterfaceData, 0, 0, out nRequiredSize, 0);
            if (!b)
            {
                if (nRequiredSize > PATH_BUFFER_SIZE)
                    throw new ApplicationException("Required buffer for device path exceeds fixed size - " + nRequiredSize.ToString());

                DeviceInterfaceDetailData.DevicePath = new string((char)0, (int)nRequiredSize);
                b = SetupDiGetDeviceInterfaceDetailW(hDeviceInfoSet, ref DeviceInterfaceData, ref DeviceInterfaceDetailData, nRequiredSize, out nRequiredSize, ref DeviceInfoData);
            }
            return b;
        }

        [DllImport("setupapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public static extern bool SetupDiGetDeviceRegistryProperty(int DeviceInfoSet, ref PSP_DEVINFO_DATA DeviceInfoData, uint Property, 
                                                                    out UInt32 PropertyRegDataType, StringBuilder PropertyBuffer, int PropertyBufferSize, out UInt32 RequiredSize);
	    public static String GetDeviceRegistryProperty( int hDevInfo, [In,Out] PSP_DEVINFO_DATA pDeviceInfoData, uint dwProperty )
	    {
            UInt32 RequiredSize;
            UInt32 RegType;
            StringBuilder sb = new StringBuilder(0);

            while (!SetupDiGetDeviceRegistryProperty(hDevInfo, ref pDeviceInfoData, dwProperty, out RegType, sb, sb.Capacity, out RequiredSize))
		    {
                int nErr = Marshal.GetLastWin32Error();

                if ( nErr == ERROR_INSUFFICIENT_BUFFER)
			    {
                    sb.Capacity = (int)RequiredSize;
                }
			    else
			    {
				    // Insert error handling here.
				    break;
			    }
		    }

            return sb.ToString();
	    }

    }
}
